-- Underveis har jeg skrevet p 1, 2, 3 osv. Dette er nummer som sier rekkeflgen ting skal kjres i
-- nr man kjrer statementsene i denne fila sammen med DemosConn2.sql. Nummerne sier hvordan man hopper
-- fram og tilbake mellom filene.

-- Enkel transaksjon ********************
USE Bank
EXEC usp_Init
SELECT * FROM Konto
BEGIN TRAN
	UPDATE Konto SET Saldo = Saldo - 100 WHERE KontoId = 3
	UPDATE Konto SET Saldo = Saldo + 100 WHERE KontoId = 1
COMMIT TRAN
--ROLLBACK TRAN

-- DDL i transaksjoner ******************
USE Bank
BEGIN TRAN
	CREATE TABLE Foo (bar int)
	SELECT * FROM Foo
ROLLBACK TRAN
SELECT * FROM Foo

-- Consistencty ******************
UPDATE Konto SET Saldo = -100

-- Effekter av samtidighet ******************
-- Conn 1
EXEC usp_Init
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
BEGIN TRAN
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 1. Initial read	
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 3. Dirty, non-rep
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 5. Phantom
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 7. After rollback
COMMIT TRAN

-- Isolasjonsniver ******************
-- Conn 1
--** Read Committed
EXEC usp_Init
SET TRANSACTION ISOLATION LEVEL READ COMMITTED
BEGIN TRAN
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 1. Initial read
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 3. Not Dirty (blokkerer siden den andre har exclusive ls), non-repeatable
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 6. Phantom
COMMIT TRAN

--** Repeatable Read
EXEC usp_Init
SET TRANSACTION ISOLATION LEVEL REPEATABLE READ
BEGIN TRAN
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 1. Initial
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 3. Repeatable read
COMMIT TRAN
-- Fr fortsatt phantoms
BEGIN TRAN
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 4. Initial
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 6. Phantom
COMMIT TRAN

--** Serializable
EXEC usp_Init
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
BEGIN TRAN
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 1. Initial
	SELECT * FROM Konto WHERE Navn = 'Nissen' -- 3. No Phantoms
COMMIT TRAN -- 4. Commit
SELECT * FROM Konto WHERE Navn = 'Nissen' -- 5. Der kommer recorden som ble satt inn. Ikke phantom siden trans. er slutt.

-- Store og sm kunder i banken ******************
-- Kjr denne og s koden i Conn2 kjapt etter hverandre. Fr da feil
-- resultat (Gran telles to ganger).
SET TRANSACTION ISOLATION LEVEL READ COMMITTED

EXEC usp_Init
DECLARE @SmaaKunder int = 0, @StoreKunder int = 0

	SELECT * FROM Konto	
	SELECT @SmaaKunder = SUM(Saldo)
	FROM Konto WHERE Saldo < 1000

	WAITFOR DELAY '00:00:5'
	
	SELECT * FROM Konto	
	SELECT @StoreKunder = SUM(Saldo)
	FROM Konto WHERE Saldo >= 1000

SELECT	@SmaaKunder AS Smaa,
		@StoreKunder AS Store,
		@SmaaKunder + @StoreKunder As Total


-- Med Repeatable read (alt bra)
EXEC usp_Init
DECLARE @SmaaKunder int = 0, @StoreKunder int = 0
SET TRANSACTION ISOLATION LEVEL REPEATABLE READ
BEGIN TRAN
	SELECT * FROM Konto	
	SELECT @SmaaKunder = SUM(Saldo)
	FROM Konto WHERE Saldo < 1000
	
	WAITFOR DELAY '00:00:5'

	SELECT * FROM Konto	
	SELECT @StoreKunder = SUM(Saldo)
	FROM Konto WHERE Saldo >= 1000
COMMIT TRAN

SELECT	@SmaaKunder AS Smaa,
		@StoreKunder AS Store,
		@SmaaKunder + @StoreKunder As Total

-- Snapshot ******************
-- Samme med snapshot isolation som viser at vi ikke trenger  blokkere
-- oppdateringstransaksjonen.
ALTER DATABASE Bank
	SET ALLOW_SNAPSHOT_ISOLATION ON

EXEC usp_Init

DECLARE @SmaaKunder int = 0, @StoreKunder int = 0

SET TRANSACTION ISOLATION LEVEL SNAPSHOT
BEGIN TRAN
	SELECT * FROM Konto
	SELECT @SmaaKunder = SUM(Saldo)
	FROM Konto WHERE Saldo < 1000
	
	WAITFOR DELAY '00:00:5'
	
	SELECT * FROM Konto
	SELECT @StoreKunder = SUM(Saldo)
	FROM Konto WHERE Saldo >= 1000
COMMIT TRAN

SELECT	@SmaaKunder AS Smaa,
		@StoreKunder AS Store,
		@SmaaKunder + @StoreKunder As Total

-- Deadlocks ******************
-- Framprovoserer deadlock
SET TRANSACTION ISOLATION LEVEL READ COMMITTED
EXEC usp_Init

CREATE TABLE Foo (Col1 int)
CREATE TABLE Bar (Col1 int)

-- Conn 1
-- 1. Begin-insert
BEGIN TRAN
	INSERT Foo VALUES (1)
	-- 3. Select. Gir deadlock.
	SELECT * FROM Bar
ROLLBACK TRAN
 
-- DBCC TRACEON(1204, 1224, 3605, -1)
-- SET DEADLOCK_PRIORITY NORMAL



